type QueueType {
  
  state continuous number q := 0;  // queue size
  input continuous number r := 0;  // incoming rate

  state CongestionControllerType controller;

  state number B    := 1;             // Bandwidth (parameter)
  state number Qmax := 10;            // Maximum queue size (parameter)

  discrete
    empty_queue { q = 0; }
    full_queue  { q = Qmax; }
    normal      { q' = r - B; }

  transition
    empty_queue -> normal {}       when { r > B },
    normal -> empty_queue {}       when { q <= 0 and r <= B ),
    normal -> full_queue  { controller:drop } when { q >= Qmax and r>B ),
    full_queue -> normal  {}       when { r < B };
}

type CongestionControllerType {
  output continuous number r := 0;    // incoming rate

  export drop;                        // synchronization event 

  discrete
     additive_increase   { r' = 1; }

  transition
     additive_increase -> additive_increase { drop } do { r = r/2; }
}

global QueueType                queue;
global CongestionControllerType contr;

setup
  define {
    contr := create(CongestionControllerType);
    queue := create(QueueType, controller:=contr); 
  }
  connect {
    r(queue) <- r(contr);
  }
