type QueueType {
  
  state continuous number q := 0;  // queue size
  state continuous number r := 0;  // total incoming rate

  input continuous number r1 := 0;  // incoming rates
  input continuous number r2 := 0;  // incoming rates

  state set(CongestionControllerType) controllers;

  state number B    := 1;             // Bandwidth (parameter)
  state number Qmax := 10;            // Maximum queue size (parameter)

  discrete
    empty_queue { q = 0;      r = r1+r2; }
    full_queue  { q = Qmax;   r = r1+r2; }
    normal      { q' = r - B; r = r1+r2; }

  transition
    empty_queue -> normal {}       when { r > B },
    normal -> empty_queue {}       when { q <= 0 and r <= B ),
    normal -> full_queue  { controllers:drop(all) } when { q >= Qmax and r>B ),
    full_queue -> normal  {}       when { r < B };
}

type CongestionControllerType {
  output continuous number r := 0;    // incoming rate

  export drop;                        // synchronization event 

  discrete
     additive_increase   { r' = 1; }

  transition
     additive_increase -> additive_increase { drop } do { r = r/2; }
}

global QueueType                queue;
global CongestionControllerType contr1;
global CongestionControllerType contr2;

setup
  define {
    contr1 := create(CongestionControllerType, r:=1);
    contr2 := create(CongestionControllerType, r:=2);
    queue := create(QueueType, controllers:={contr1,contr2}); 
  }
  connect {
    r1(queue)<-r(contr1);
    r2(queue)<-r(contr2);
  }